#!/usr/bin/perl -w
#< Boilerplate
#
# The Aranea question answering system
# Copyright (C) 2002-2005, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit
# http://www.gnu.org/copyleft/gpl.html
#
# $Header: /fs/clip-qa/.cvsroot//Aranea/pm/Aranea/Modules/Combine.pm,v 1.12 2005/06/10 16:17:16 jimmylin Exp $
#
#>
#< POD
###############################################################################

=head1 NAME

B<C<Aranea::Modules::Combine>> - Aranea module for adjusting the score
of answers that are substrings of each other

=head1 SYNOPSIS

This Aranea modules is meant to be used in conjunction with the
L<C<Aranea::Shell>|Aranea::Shell>.

=head1 DESCRIPTION

This Aranea module boosts the score of a candidate if its substrings
also appear as answer candidates.  For example, the score of candidate
"A B" will be increased if both "A" and "B" appear as candidates.

=cut

###############################################################################
#>
#< Header
package Aranea::Modules::Combine;

use strict;
use warnings;

use Data::Dumper;
use IO::Handle;

use Aranea::XML::Aranea;

#>

#< STDIN handler
unless ( caller() ) {
    my $stdin = new_from_fd IO::Handle(fileno(STDIN),"r");
    my $Aranea = Aranea::XML::Aranea->new($stdin);

    $Aranea = doModule($Aranea);

    $Aranea->write_xml();
}
#>

#< sub doModule
sub doModule {
    my $Aranea = shift;

    $Aranea->insert_pipeline_tag('$Id: Combine.pm,v 1.12 2005/06/10 16:17:16 jimmylin Exp $ ');

    # if there are no requests, then there is nothing to do
    return $Aranea if $Aranea->request_list()->entries_count() == -1;

    my $entries = $Aranea->candidate_list()->entries();
    #print Dumper($entries);
    my %candidate_hash = ();

    # put the entries into a hash such that the key is the candidate
    # answer and the value is the entry reference.

    for my $entry (@$entries) {
	$candidate_hash{$entry->candidate()} = $entry;
    }

    # now loop through each entry, split on whitespace, and see if
    # both values are in the hash.

    for my $entry (@$entries) {

	my @words = split(/\s+/, $entry->candidate());

	# if there was only one word, skip it.
	next if $#{@words} == 0;

	# now see if all the words that make up this candidate answer
	# are in our hash somewhere.

	for my $word (@words) {
	    my $hashed_word = $candidate_hash{$word};
	    $entry->score($entry->score() + $hashed_word->score())
		if $hashed_word;
	}
    }

    return $Aranea;
}
#>

1;
