#!/usr/bin/perl -w
#< Boilerplate
#
# The Aranea question answering system
# Copyright (C) 2002-2005, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit
# http://www.gnu.org/copyleft/gpl.html
#
# $Header: /fs/clip-qa/.cvsroot//Aranea/pm/Aranea/Modules/GenerateBackoffRequest.pm,v 1.13 2005/06/10 16:17:16 jimmylin Exp $
#
#>
#< POD
###############################################################################

=head1 NAME

B<C<Aranea::Modules::GenerateQuery>> - Aranea module for generating
the initial request

=head1 SYNOPSIS

This Aranea modules is meant to be used in conjunction with the
L<C<Aranea::Shell>|Aranea::Shell>.

=head1 DESCRIPTION

This Aranea module populates the C<request_list> with a single default
"backoff" request, which is merely an inexact request containing all
the words in the question.

=cut

###############################################################################
#>
#< Header
package Aranea::Modules::GenerateBackoffRequest;

use strict;
use warnings;
use lib qw(lib pm);                         # needed for command-line mode

use IO::Handle;

use Aranea::XML::Aranea;

our $inexact_snippets_count = 100;

#>

#< STDIN handler
unless ( caller() ) {
    my $stdin = new_from_fd IO::Handle(fileno(STDIN),"r");
    my $Aranea = Aranea::XML::Aranea->new($stdin);

    $Aranea = doModule($Aranea);

    $Aranea->write_xml();
}
#>

#< sub doModule
sub doModule {
    my $Aranea = shift;

    $Aranea->insert_pipeline_tag('$Id: GenerateBackoffRequest.pm,v 1.13 2005/06/10 16:17:16 jimmylin Exp $ ');

    # if we have the answer already (e.g., via database lookup), then
    # don't bother generating requests.
    return $Aranea if $Aranea->candidate_list()->entries_count() != -1;

    my $q = $Aranea->query()->text();
    my $qid = $Aranea->query()->id();

    $Aranea->request_list()
	->add_default_inexact_query($q, $inexact_snippets_count);

    return $Aranea;
}
#>

#< set_inexact_snippets_count
sub set_inexact_snippets_count {
    $inexact_snippets_count = shift;
}
#>

1;
