#!/usr/bin/perl -w
#< Boilerplate
#
# The Aranea question answering system
# Copyright (C) 2002-2005, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit
# http://www.gnu.org/copyleft/gpl.html
#
# $Header: /fs/clip-qa/.cvsroot//Aranea/pm/Aranea/Modules/LookupPerson.pm,v 1.9 2005/06/10 16:17:16 jimmylin Exp $
#
#>
#< POD
###############################################################################

=head1 NAME

B<C<Aranea::Modules::LookupPerson>> - Aranea module for answering
location questions using database techniques

=head1 SYNOPSIS

This Aranea modules is meant to be used in conjunction with the
L<C<Aranea::Shell>|Aranea::Shell>.

=head1 DESCRIPTION

This module answers the following types of person questions:

=over 4

=item *

The I<n>-th president, i.e., Who was the 22th president? (from POTUS)

=item *

The president during a particular year, i.e. Who was president during
1812? (from POTUS)

=back

=cut

###############################################################################
#>
#< Header
package Aranea::Modules::LookupPerson;

use strict;
use warnings;

use IO::Handle;

use Aranea::XML::Aranea;
use Aranea::Resources::Gateway;

#>

#< STDIN handler
unless ( caller() ) {
    my $stdin = new_from_fd IO::Handle(fileno(STDIN),"r");
    my $Aranea = Aranea::XML::Aranea->new($stdin);

    $Aranea = doModule($Aranea);

    $Aranea->write_xml();
}
#>

#< sub doModule
sub doModule {
    my $Aranea = shift;

    $Aranea->insert_pipeline_tag('$Id: LookupPerson.pm,v 1.9 2005/06/10 16:17:16 jimmylin Exp $ ');

    my $query = $Aranea->query()->text();
    my $arg;

    db_insert($Aranea, "POTUS", ["ordinality", $arg])
	if $arg = &pres_by_ord_sig($query);

    db_insert($Aranea, "POTUS", ["year", $arg])
	if $arg = &pres_by_year_sig($query);

    return $Aranea;
}
#>

#< sub pres_by_ord_sig
sub pres_by_ord_sig {
    my $s = shift;

    if ( $s =~ /president/i and $s =~ /(United States|US|U\.S\.)/) {
	return $1 if $s =~ /(\d\d)(st|nd|rd|th)/;

	return 1 if ( $s =~ /first/ and 
		      $s =~ /first (U\.?S\.? )?president\??$/ );
	# make sure we are matching a complete NP with
	# nothing following it

	# be careful of too aggressively matching 'first U.S. president'
	# e.g.,
	#   273, Who was the first U.S. president ever to resign?
	#  1210, Who was the first U.S. president to appear on TV?
	#  1275, Who was the first vice president of the U.S.?
	#  1277, Who was the first US President to ride in an
	#        automobile to his inauguration?

	return 2 if $s =~ /second/;
	return 3 if $s =~ /third/;
	return 4 if $s =~ /fourth/;
	return 5 if $s =~ /fifth/;
	return 6 if $s =~ /sixth/;
	return 7 if $s =~ /seventh/;
	return 8 if $s =~ /eighth/;
	return 9 if $s =~ /ninth/;
	return 10 if $s =~ /tenth/;
    }

    return '';
}
#>
#< sub pres_by_year_sig
sub pres_by_year_sig {
    my $s = shift;

    if ( $s =~ /Who was president in \d\d\d\d\?/i or
	 ( $s =~ /president/ and $s =~ /(United States|US|U\.S\.)/ ) ) {
	return $& if $s =~ /\d\d\d\d/;
    }

    return '';
}
#>

1;
