#!/usr/bin/perl -w
#< Boilerplate
#
# The Aranea question answering system
# Copyright (C) 2002-2005, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit
# http://www.gnu.org/copyleft/gpl.html
#
# $Header: /fs/clip-qa/.cvsroot//Aranea/pm/Aranea/Patterns/Dates.pm,v 1.5 2005/06/08 02:51:43 jimmylin Exp $
#
#>
#< Header
package Aranea::Patterns::Dates;

use strict;
use warnings;

use Exporter;

our @ISA = qw(Exporter);
our @EXPORT = qw();
our @EXPORT_OK = qw($YEARS_AGO $AD_BC $MONTH $MONTH_ABBREV $MONTH_FULL $DAY $YEAR $CENTURY $DATE %ABBREV_TO_MONTH %MONTH_TO_ABBREV);

#>

#< regexp: YEARS_AGO
# n years ago
our $YEARS_AGO = qr{[\d,.]+(?: (?:thousand|million|billion|trillion))? years(?: ago)?}o;
#>
#< regexp: AD_BC
# various ways of saying AD and BC
our $AD_BC = qr{(?:AD|BC|ADE|BCE|A\.D\.|B\.C\.|A\.D\.E|B\.C\.E\.)}o;
#>
#< regexp: MONTH_ABBREV
our $MONTH_ABBREV = qr{(?:Jan\.?|Feb\.?|Mar\.?|Apr\.?|Jun\.?|Jul\.?|Aug\.?|Sept\.?|Oct\.?|Nov\.?|Dec\.?|JAN\.?|FEB\.?|MAR\.?|APR\.?|JUN\.?|JUL\.?|AUG\.?|SEPT\.?|OCT\.?|NOV\.?|DEC\.?)}o;
#>
#< regexp: MONTH_FULL
our $MONTH_FULL = qr{(?:January|February|March|April|May|June|July|August|September|October|November|December)}o;
#>
#< regexp: MONTH
# all the months and their abbreviations
our $MONTH = qr{(?:$MONTH_FULL|$MONTH_ABBREV)}o;
#>
#< regexp: DAY
# days, include variants like '15th-22th'
our $DAY = qr{\d+(?:st|nd|rd|th)?(?:\s*-\s*\d+(?:st|nd|rd|th)?)?}o;
#>
#< regexp: YEAR
# years, in various forms
our $YEAR = qr{(?:\d+s?(?:\s+$AD_BC)?|    # e.g., 79 AD or 1930s
                 $AD_BC\s+\d+)            # e.g., AD 79
             }xo;
#>
#< regexp: CENTURY
# nth century
our $CENTURY = qr{\d+(?:st|nd|rd|th) century(?: $AD_BC)?}o;
#>
#< regexp: DATE
# all the various ways dates could be expressed
our $DATE = qr{^(?:$YEAR|
                  $YEAR\s*-+\s*$YEAR|
                  $YEARS_AGO|
                  $CENTURY|
                  $DAY\s+$MONTH|
                  $MONTH\s$DAY|
                  (?:$DAY\s+)$MONTH,?\s+$YEAR|
	          $MONTH\s+$DAY,\s+$YEAR)$}xo;
#>

#< hash: ABBREV_TO_MONTH
our %ABBREV_TO_MONTH = (
  'Jan' => [ 'January' ],
  'Feb' => [ 'February' ],
  'Mar' => [ 'March' ],
  'Apr' => [ 'April' ],
  'Jun' => [ 'June' ],
  'Jul' => [ 'July' ],
  'Aug' => [ 'August' ],
  'Sept' => [ 'September' ],
  'Oct' => [ 'October' ],
  'Nov' => [ 'November' ],
  'Dec' => [ 'December' ],
  'Jan.' => [ 'January' ],
  'Feb.' => [ 'February' ],
  'Mar.' => [ 'March' ],
  'Apr.' => [ 'April' ],
  'Jun.' => [ 'June' ],
  'Jul.' => [ 'July' ],
  'Aug.' => [ 'August' ],
  'Sept.' => [ 'September' ],
  'Oct.' => [ 'October' ],
  'Nov.' => [ 'November' ],
  'Dec.' => [ 'December' ],
  'JAN' => [ 'January' ],
  'FEB' => [ 'February' ],
  'MAR' => [ 'March' ],
  'APR' => [ 'April' ],
  'JUN' => [ 'June' ],
  'JUL' => [ 'July' ],
  'AUG' => [ 'August' ],
  'SEPT' => [ 'September' ],
  'OCT' => [ 'October' ],
  'NOV' => [ 'November' ],
  'DEC' => [ 'December' ],
  'JAN.' => [ 'January' ],
  'FEB.' => [ 'February' ],
  'MAR.' => [ 'March' ],
  'APR.' => [ 'April' ],
  'JUN.' => [ 'June' ],
  'JUL.' => [ 'July' ],
  'AUG.' => [ 'August' ],
  'SEPT.' => [ 'September' ],
  'OCT.' => [ 'October' ],
  'NOV.' => [ 'November' ],
  'DEC.' => [ 'December' ]
);
#>
#< hash: MONTH_TO_ABBREV

# We purposely don't want to add the trailing dot because it's
# optional and we usally want to match both versions... instead, let
# the regular expression handle matching the optional dot, e.g.,
# /Jan\.?/

our %MONTH_TO_ABBREV = (
    'January'   => 'Jan',
    'February'  => 'Feb',
    'March'     => 'Mar',
    'April'     => 'Apr',
    'May'       => 'May',
    'June'      => 'June',
    'July'      => 'Jul',
    'August'    => 'Aug',
    'September' => 'Sept',
    'October'   => 'Oct',
    'November'  => 'Nov',
    'December'  => 'Dec',
);
#>

1;
