# LPost - Brill's Part-of-Speech Tagger as a Perl Module
# Copyright (C) 2003-2004, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit 
# http://www.gnu.org/copyleft/gpl.html

package LPost;

use 5.006;
use strict;
use warnings;
use Errno;
use Carp;

require Exporter;
require DynaLoader;
use AutoLoader;

our @ISA = qw(Exporter DynaLoader);

our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw();

our $VERSION = '0.01';

sub AUTOLOAD {
    # This AUTOLOAD is used to 'autoload' constants from the constant()
    # XS function.  If a constant is not found then control is passed
    # to the AUTOLOAD in AutoLoader.

    my $constname;
    our $AUTOLOAD;
    ($constname = $AUTOLOAD) =~ s/.*:://;
    croak "& not defined" if $constname eq 'constant';
    my $val = constant($constname, @_ ? $_[0] : 0);
    if ($! != 0) {
	if ($!{EINVAL}) {
	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
	    goto &AutoLoader::AUTOLOAD;
	}
	else {
	    croak "Your vendor has not defined LPost macro $constname";
	}
    }
    {
	no strict 'refs';
	# Fixed between 5.005_53 and 5.005_61
	if ($] >= 5.00561) {
	    *$AUTOLOAD = sub () { $val };
	}
	else {
	    *$AUTOLOAD = sub { $val };
	}
    }
    goto &$AUTOLOAD;
}

bootstrap LPost $VERSION;

sub initialize_tagger {
    # the real initialization of the tagger is performed in C, but we
    # have to find the lexicon files.  If the tagger was properly
    # installed, the files should be somewhere in @INC.

    my $basepath;

    foreach my $path ( @INC ) {
	if ( -e "$path/LEXICON" ) {
	    $basepath = $path;
	    last;
	}
    }

    # call the real initializer in C
    _initialize($basepath);
}

1;
__END__

=head1 NAME

LPost - Brill's Part-of-Speech Tagger as a Perl Module

=head1 SYNOPSIS

  use LPost;
  LPost::initialize_tagger();
  my $s = LPost::tag('This is a test of the part-of-speech tagger');
  LPost::destroy_tagger();

=head1 DESCRIPTION

Eric Brill's part-of-speech tagger as a Perl Module.  It's based on
Benjamin Han's ePost package at
L<http://www-2.cs.cmu.edu/~benhdj/Code/>, which is a cleaned-up
version of Brill's original code.

=head1 AUTHOR

Jimmy Lin (jimmylin at umd.edu)

=head1 HISTORY

Version 1.0 (December 27, 2004)

=head1 LICENSE

Copyright (C) 2002-2004, Jimmy Lin

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, please visit 
http://www.gnu.org/copyleft/gpl.html

=cut
